@@ -1,4 +1,3 @@
-package org.aspectj.ajde;
 /* *******************************************************************
  * Copyright (c) 1999-2001 Xerox Corporation, 
  *               2002 Palo Alto Research Center, Incorporated (PARC).
@@ -11,9 +10,7 @@ package org.aspectj.ajde;
  * Contributors: 
  *     Xerox/PARC     initial implementation 
  * ******************************************************************/
-
-
-// default package
+package org.aspectj.ajde;
 
 import junit.framework.*;
 
@@ -22,8 +19,6 @@ public class AjdeModuleTests extends TestCase {
     public static TestSuite suite() { 
         TestSuite suite = new TestSuite(AjdeModuleTests.class.getName());
         suite.addTest(org.aspectj.ajde.AjdeTests.suite()); 
-        suite.addTest(org.aspectj.ajde.internal.AjdeInternalTests.suite()); 
-        suite.addTest(org.aspectj.ajde.ui.AjdeUiTests.suite()); 
         return suite;
     }
 
