@@ -16,22 +16,21 @@ package org.aspectj.testing.harness.bridge;
 import java.io.File;
 import java.io.FileFilter;
 import java.io.IOException;
-//import java.util.*;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
-//import java.util.Collections;
-//import java.util.List;
 
-import org.aspectj.ajde.ui.StructureModelUtil;
-import org.aspectj.ajde.ui.StructureModelUtil.ModelIncorrectException;
+import org.aspectj.bridge.AbortException;
 import org.aspectj.bridge.ICommand;
-//import org.aspectj.bridge.IMessage;
 import org.aspectj.bridge.MessageUtil;
 import org.aspectj.testing.ajde.CompileCommand;
 import org.aspectj.testing.run.IRunIterator;
 import org.aspectj.testing.run.IRunStatus;
 import org.aspectj.testing.run.WrappedRunIterator;
+import org.aspectj.testing.util.StructureModelUtil;
+import org.aspectj.testing.util.StructureModelUtil.ModelIncorrectException;
+import org.aspectj.testing.xml.AjcSpecXmlReader;
+import org.aspectj.testing.xml.IXmlWritable;
 import org.aspectj.testing.xml.SoftMessage;
 import org.aspectj.testing.xml.XMLWriter;
 import org.aspectj.util.FileUtil;
