@@ -64,6 +64,7 @@ import org.aspectj.weaver.AsmRelationshipProvider;
 import org.aspectj.weaver.BCException;
 import org.aspectj.weaver.ConcreteTypeMunger;
 import org.aspectj.weaver.CrosscuttingMembersSet;
+import org.aspectj.weaver.CustomMungerFactory;
 import org.aspectj.weaver.IClassFileProvider;
 import org.aspectj.weaver.IWeaveRequestor;
 import org.aspectj.weaver.IWeaver;
@@ -138,6 +139,7 @@ public class BcelWeaver implements IWeaver {
 	private List declareParentsList = null; // setup by prepareForWeave
 
     private ZipOutputStream zipOutputStream;
+	private CustomMungerFactory customMungerFactory;
 
 	// ----
     
@@ -488,6 +490,8 @@ public class BcelWeaver implements IWeaver {
 		typeMungerList = xcutSet.getTypeMungers();
         lateTypeMungerList = xcutSet.getLateTypeMungers();
 		declareParentsList = xcutSet.getDeclareParents();
+		
+		addCustomMungers();
     	
 		// The ordering here used to be based on a string compare on toString() for the two mungers - 
 		// that breaks for the @AJ style where advice names aren't programmatically generated.  So we
@@ -519,6 +523,30 @@ public class BcelWeaver implements IWeaver {
 		if (trace.isTraceEnabled()) trace.exit("prepareForWeave");
     }
     
+    private void addCustomMungers() {
+		if (customMungerFactory != null) {
+			for (Iterator i = addedClasses.iterator(); i.hasNext();) {
+				UnwovenClassFile jc = (UnwovenClassFile) i.next();
+				String name = jc.getClassName();
+				ResolvedType type = world.resolve(name);
+				if (type.isAspect()) {
+					Collection/*ShadowMunger*/ shadowMungers = customMungerFactory.createCustomShadowMungers(type);
+					if (shadowMungers != null) {
+						shadowMungerList.addAll(shadowMungers);
+					}
+					Collection/*ConcreteTypeMunger*/ typeMungers = customMungerFactory
+							.createCustomTypeMungers(type);
+					if (typeMungers != null)
+						typeMungerList.addAll(typeMungers);
+				}
+			}
+		}
+	}
+    
+    public void setCustomMungerFactory(CustomMungerFactory factory) {
+    	customMungerFactory = factory;
+    }
+    
     /*
      * Rewrite all of the pointcuts in the world into their most efficient
      * form for subsequent matching. Also ensure that if pc1.equals(pc2)
@@ -1035,6 +1063,17 @@ public class BcelWeaver implements IWeaver {
 	        }
     	   }
 
+    	// Go through the types and ensure any 'damaged' during compile time are repaired prior to weaving
+    	for (Iterator i = input.getClassFileIterator(); i.hasNext(); ) {
+            UnwovenClassFile classFile = (UnwovenClassFile)i.next();
+            String className = classFile.getClassName();
+            ResolvedType theType = world.resolve(className);
+            if (theType!=null) {
+	            BcelObjectType classType = BcelWorld.getBcelObjectType(theType);
+	            if (classType!=null) classType.ensureDelegateConsistent();
+            }
+    	}
+
         // special case for AtAspectJMungerOnly - see #113587
         if (input.isApplyAtAspectJMungersOnly()) {
             ContextToken atAspectJMungersOnly = CompilationAndWeavingContext.enteringPhase(CompilationAndWeavingContext.PROCESSING_ATASPECTJTYPE_MUNGERS_ONLY, "");
