@@ -1237,7 +1237,8 @@ public final class LazyClassGen {
 	}
 	
 	private void makeSyntheticAndTransientIfNeeded(Field field) {
-		if (field.getName().startsWith(NameMangler.PREFIX)) {
+		if (field.getName().startsWith(NameMangler.PREFIX) &&
+			!field.getName().startsWith("ajc$interField$")) {
 			// it's an aj added field
 			// first do transient
 			if (!field.isStatic()) {
@@ -1320,8 +1321,10 @@ public final class LazyClassGen {
 	}
 	
 	
-	public void forcePublic() {
+	public boolean forcePublic() {
+		if (myGen.isPublic()) return false;
 		myGen.setAccessFlags(Utility.makePublic(myGen.getAccessFlags()));
+		return true;
 	}
 
 	
